if not RGM or not RGM.Build then return end

local Gsub, Lower, Match, String, Upper = string.gsub, string.lower, string.match, string.format, string.upper
local E, G, I, S, T, F = RGM, RGM_Global, RGM.Interface[1], RGM.Interface[2], true, false
local Register, Inspect, Number = Command.Event.Attach, Inspect, tonumber
local Insert, Remove = table.insert, table.remove
local Integer, Sort = math.floor, table.sort
local Next, Pairs = next, pairs

function E.Commands(Debug, Input)
	if not Input or not Match(Input, "%a+") then return end

	local I, S, Run = RGM.Interface[1], RGM.Interface[2], T

	if Input == "Short 1" or Input == "short 1" then
		G[1].Short = not G[1].Short
	end

	if Input == "Short 2" or Input == "short 2" then
		G[2].Short = not G[2].Short
	end

	if Input == "Force" or Input == "force" then
		G.Force = not G.Force
	end

	if Input and Run then
		Run = I and I.Update(I.Session(O, 0))
		Run = S and S.Update(S.Session(O, 0))
	end
end

function E.Force(Input)
	if not Input or not G.Force then return end

	local Data, Unit, Run = Input.caster and Input.target, F, F

	if Data then
		local Owner = Inspect.Unit.Lookup(Input.caster .. ".owner")
		local Lookup = Inspect.Unit.Lookup(Input.caster)
		
		if Lookup and Match(Lookup, "focus") or Lookup and Match(Lookup, "mouseover") then
			return
		end
		
		if Owner and Match(Owner, "focus") or Owner and Match(Owner, "mouseover") then
			return
		end
		
		if Lookup and Match(Lookup, "group") and not Match(Lookup, ".") then
			Run = T
		end
		
		if Owner and Match(Owner, "group") and not Match(Owner, ".") then
			Run = T
		end
		
		if Input.caster == E.User or Owner == E.User then
			Run = T
		end
		
		if Run then
			Unit = Inspect.Unit.Detail(Input.target)
			Data = {[Input.caster] = T}
		end
	end

	if Data and Unit and Unit.combat then
		E.Combat(Input, Data)
	end
end

function E.Gather(Input, Receive)
	local Ability, Count, Search, Sync = E.Ability, E.Count, E.Search, E.Sync
	local Caster, Target = {Abilities = {}}, {Abilities = {}}
	local Input, Given, Taken = Input, 0, 0

	if Caster then
		Caster = (Search(Input.casterName, Input.caster) or Given and Caster)
	end

	if Target then
		Target = (Search(Input.targetName, Input.target) or Taken and Target)
	end

	if not Input.converted and not Receive then
		if Caster.Blocked and Sync(Caster.Blocked, Input, Input.caster) then
			return
		end
		
		if Target.Blocked and Sync(Target.Blocked, Input, Input.target) then
			return
		end
	end

	if Caster.Abilities then
		Input.casterName = (Caster.Name or Input.casterName)
		Given = Ability(Input, "Given", Caster)
	end

	if Target.Abilities then
		Input.targetName = (Target.Name or Input.targetName)
		Taken = Ability(Input, "Taken", Target)
	end

	if Input.damage then
		Count(Given.Damage, "damage", Input, Target.Name, "Given", "Damage")
		Count(Taken.Damage, "damage", Input, Caster.Name, "Taken", "Damage")
	end

	if Input.damageAbsorbed then
		Count(Given.Absorbed, "damageAbsorbed", Input, Target.Name, "Given", "Absorb")
		Count(Taken.Absorbed, "damageAbsorbed", Input, Caster.Name, "Taken", "Absorb")
	end

	if Input.damageAvoided then
		Count(Given.Missed, "damageAvoided", Input, Target.Name, "Given", "Miss")
		Count(Taken.Avoided, "damageAvoided", Input, Caster.Name, "Taken", "Avoid")
	end

	if Input.damageBlocked then
		Count(Given.Blocked, "damageBlocked", Input, Target.Name, "Given", "Block")
		Count(Taken.Blocked, "damageBlocked", Input, Caster.Name, "Taken", "Block")
	end

	if Input.damageIntercepted then
		Count(Given.Intercepted, "damageIntercepted", Input, Target.Name, "Given", "Intercept")
		Count(Taken.Intercepted, "damageIntercepted", Input, Caster.Name, "Taken", "Intercept")
	end

	if Input.heal then
		Count(Given.Heal, "heal", Input, Target.Name, "Given", "Heal")
		Count(Taken.Heal, "heal", Input, Caster.Name, "Taken", "Heal")
	end

	if Input.overheal then
		Count(Given.Overheal, "overheal", Input, Target.Name, "Given", "Overheal")
		Count(Taken.Overheal, "overheal", Input, Caster.Name, "Taken", "Overheal")
	end

	if Input.overkill then
		Count(Given.Overkill, "overkill", Input, Target.Name, "Given", "Overkill")
		Count(Taken.Overkill, "overkill", Input, Caster.Name, "Taken", "Overkill")
	end

	if Target.Role then
		if Input.damage or Input.damageAbsorbed or Input.heal then
			E.Log(Input, Target)
		end
		
		if Input.damageAbsorbed then
			E.Converter(Input)
		end
	end
end

function E.Processor(Avoid, Input)
	local Input = E.Private(Input) if not Input then return end

	local Convert = (Input.damageAbsorbed or Input.damageIntercepted or Input.damageBlocked)
	local Event = (Avoid or Input.damage or Input.damageAbsorbed)
	local Self = (Input.caster == Input.target)
	local Heal = not (Convert or Event)

	if Convert and Self or Event and Self then
		return
	end

	if not E.Active and not Heal then
		E.Force(Input)
	end

	if E.Active then
		Input.damageIntercepted = (Input.damageIntercepted and Input.damageIntercepted < 0 and -Input.damageIntercepted or Input.damageIntercepted)
		Input.ability, Input.abilityNew, Input.damageAvoided, Input.type = O, O, Avoid, (Input.type and Gsub(Input.type, "^%l", Upper))
		Input.damageAbsorbed = (Input.damageAbsorbed and Input.damageAbsorbed < 0 and -Input.damageAbsorbed or Input.damageAbsorbed)
	else
		return
	end

	if Convert then
		Input.damage = (Input.damage or 0) + (Input.damageAbsorbed or 0) + (Input.damageIntercepted or 0) + (Input.damageBlocked or 0)
		Input.taken = Input.damage
	end

	if not Input.abilityName or Match(Input.abilityName, "attack") or not Match(Input.abilityName, "%a+") then
		Input.abilityName = "Auto Attack"
	end

	if not Input.casterName or not Match(Input.casterName, "%a+") then
		Input.casterName = "Unknown"
	end

	if not Input.targetName or not Match(Input.targetName, "%a+") then
		Input.targetName = "Unknown"
	end

	if Input.damageAbsorbed and not Input.damage then
		Input.damage = 0
	end

	E.Gather(Input)
end

if E.Commands then
	Register(Command.Slash.Register("RGME"), E.Commands, "Slash")
end